//Project: LanderControl
//Homepage: www.HomoFaciens.de
//Author Norbert Heinz
//Version: 1.0
//Creation date: 26.10.2011
//Copyright: (c) 2011 by Norbert Heinz
//Microcontroller: Atmega8-16PU
//This program is free software you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation version 3 of the License.
//This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with this program if not, see http://www.gnu.org/licenses/
//
//Program controls 3 servos (movement of camera arm), one power transistor (light) and one wired and one infra red interface.
//Program based on PowerSwitch (author Christian Starkjohann) and usb-servo (author Ronald Schaten) - Thanks!
//Program uses V-USB,  published  under GNU GPL v2, GNU GPL v3 or proprietary (CommercialLicense.txt), Copyright: (c) 2005 by OBJECTIVE DEVELOPMENT Software GmbH


#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>

#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"
#include <util/delay.h>

#define EEPROM_LOCATION (void *)37
#define MODULE_ID_TO_SEND 5


static int armHeightAngle = 0;
static int verticalAngle = 0;
static int horizontalAngle = 0;
static int headLight = 0;
static int timeCounter01 = 0;
static int sendBytes = 0;
static int moduleNumber = 0;
static int moduleInstruction = 0;
static int irCommand = 0;
static int temp01 = 0;
static int moduleIRid = 0;
static int irLEDcounter = 0;
static int get01 = 0;
static int get02 = 0;
static int controlBits = 0;


usbMsgLen_t usbFunctionSetup(uchar setupData[8]){
static uchar    replyBuf[2];

  usbMsgPtr = replyBuf;
  usbRequest_t *rq = (void *)setupData;   // cast to structured data for parsing

  switch(rq->bRequest){
    case 0:                                 //ECHO
      replyBuf[0] = rq->wValue.bytes[0];
      replyBuf[1] = rq->wValue.bytes[1];
      return 2;

    case 1:                                 //Get module number for next instruction via wired connection
      moduleNumber = setupData[2];
      return 0;

    case 2:                                 //Get next instruction byte and send module number + instruction via wired interface
      moduleInstruction = setupData[2];
      sendBytes = 1;
      return 0;

    case 3:                                 //Set horizontal angle of camera
      horizontalAngle = setupData[2];
      return 0;

    case 4:                                 //Set vertical angle of camera
      verticalAngle = setupData[2];
      return 0;

    case 5:                                 //Set height of cameraposition
      armHeightAngle = setupData[2];
      return 0;

    case 6:                                 //Switch headlight
      headLight = setupData[2];
      return 0;

    case 10:                                 //Sleep
        armHeightAngle = 0;
        verticalAngle = 0;
        horizontalAngle = 0;
        headLight = 0;
      return 0;

    case 11:                                 //Send bytes via wired interface
      replyBuf[0] = get01;
      replyBuf[1] = get02;
      return 2;

  }
  return 0;                               // ignore all unknown requests
}


static void timerInterrupt(void){

  if(headLight > 0){
    PORTB = PORTB | 0b00000001;
  }

  timeCounter01++;
  if(timeCounter01 >= 5){
    timeCounter01 = 0;
    if(armHeightAngle > 0 && armHeightAngle < 180){
      PORTD |= 0b00100000;  //Turn servo pins on
    }
    if(verticalAngle > 0 && verticalAngle < 180){
      PORTC |= 0b00000001;  //Turn servo pins on
    }
    if(horizontalAngle > 0 && horizontalAngle < 180){
      PORTC |= 0b00000010;  //Turn servo pins on
    }
  }

}


static void timerInterrupt_2(void){

  if(irLEDcounter > 0){
    irLEDcounter--;
  }
}


static void readInput(void){
  temp01 = 0;
  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(PINB & (1 << 2) && PINB & (1 << 3));
  if(PINB & (1 << 1)){
    temp01 |= 0b00000001;
  }
  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(!(PINB & (1 << 2)) && PINB & (1 << 3));


  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(PINB & (1 << 2) && PINB & (1 << 3));
  if(PINB & (1 << 1)){
    temp01 |= 0b00000010;
  }
  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(!(PINB & (1 << 2)) && PINB & (1 << 3));


  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(PINB & (1 << 2) && PINB & (1 << 3));
  if(PINB & (1 << 1)){
    temp01 |= 0b00000100;
  }
  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(!(PINB & (1 << 2)) && PINB & (1 << 3));


  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(PINB & (1 << 2) && PINB & (1 << 3));
  if(PINB & (1 << 1)){
    temp01 |= 0b00001000;
  }
  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(!(PINB & (1 << 2)) && PINB & (1 << 3));


  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(PINB & (1 << 2) && PINB & (1 << 3));
  if(PINB & (1 << 1)){
    temp01 |= 0b00010000;
  }
  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(!(PINB & (1 << 2)) && PINB & (1 << 3));


  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(PINB & (1 << 2) && PINB & (1 << 3));
  if(PINB & (1 << 1)){
    temp01 |= 0b00100000;
  }
  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(!(PINB & (1 << 2)) && PINB & (1 << 3));


  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(PINB & (1 << 2) && PINB & (1 << 3));
  if(PINB & (1 << 1)){
    temp01 |= 0b01000000;
  }
  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(!(PINB & (1 << 2)) && PINB & (1 << 3));


  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(PINB & (1 << 2) && PINB & (1 << 3));
  if(PINB & (1 << 1)){
    temp01 |= 0b10000000;
  }
  if(!(PINB & (1 << 3))){
    temp01 = 0b11111111;
    return;
  }
  while(!(PINB & (1 << 2)) && PINB & (1 << 3));

  return;
}

int main(void){
uchar   i;

  //Port B, Pin 0 = Output, Transistor beamlight

  //Port B, Pin 1 = Input/Output, wired communication with other microcontrollers
  //Port B, Pin 2 = Input/Output, wired communication with other microcontrollers
  //Port B, Pin 3 = Input/Output, wired communication with other microcontrollers

  //Port B, Pin 6 = External oscillator
  //Port B, Pin 7 = External oscillator

  //Port C, Pin 0 = Output, Servo vertical camera angle
  //Port C, Pin 1 = Output, Servo horizontal angle

  //Port D, Pin 0 = Output, infrared diode
  //Port D, Pin 1 = Input, infrared sensor
  //Port D, Pin 2 = Input, USB, Int0
  //Port D, Pin 4 = Output, green signal LED
  //Port D, Pin 5 = Output, servo arm height
  //Port D, Pin 6 = Input, USB D-
  //Port D, Pin 7 = Input, USB D+

  DDRD =  0b00110001;
  PORTD = 0b00000000;  //disable pullup resitors at usb-pins
  DDRB =  0b11100001;
  PORTB = 0b00000000;  //no pullup resistors at communication pins (1-3)!
  DDRC =  0b00100011;
  PORTC = 0b00000000;  //set pins of port C to 0

  TCCR1A = 0x00;
  TCCR1B = (1<<CS12) | (1<<WGM12); //Configure Timer 0, prescaler 256
  OCR1A  = 188;                 //Interrupt after 12,000,000 / 256 / 188 = 249Hz (after 4.011ms)
  OCR1B = 188;                  //Interrupt after 12,000,000 / 256 / 188 = 249Hz (after 4.011ms)

  //Initialize timer 2 (8-bit) - IR communication
  OCR2 = 15;       //Compare value
  TCCR2 = ( 1 << WGM21 ) | ( 1 << CS22 );  //Switch to CTC mode, prescaler 64, 12 000 000/64/15 = 12500Hz = 0.08ms

  PORTB = PORTB | 0b00000001;

  usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
  i = 0;
  while(--i){             /* fake USB disconnect for > 500 ms */
    wdt_reset();
    _delay_ms(2);
  }
  usbDeviceConnect();
  usbInit();

  PORTB &= 0b11111110;

  sei();


  while(1 == 1){    /* main event loop */
    usbPoll();

    if(TCNT1 > armHeightAngle){
      PORTD &= 0b11011111;  //Turn servo pin off
    }
    if(TCNT1 > verticalAngle){
      PORTC &= 0b11111110;  //Turn servo pin off
    }
    if(TCNT1 > horizontalAngle){
      PORTC &= 0b11111101;  //Turn servo pin off
    }
    if(TCNT1 > headLight){
      PORTB &= 0b11111110;  //headlight off
    }


    if (TIFR & (1 << OCF1A)) {//'Manual' timer interrupt. Normal timer interrupt troubles USB-communication
      TIFR |= 1 << OCF1A;  /* clear pending flag */
      TCNT1 = 0;
      timerInterrupt();
    }

    if (TIFR & (1 << OCF2)) {//'Manual' timer interrupt. Normal timer interrupt troubles USB-communication
      TIFR |= 1 << OCF2;  /* clear pending flag */
      TCNT2 = 0;
      timerInterrupt_2();
    }


    //send bytes to wired connection
    if(sendBytes == 1 && !(PINB & (1 << 3))){
        PORTB &= 0b11110001;
        DDRB |= 0b00001110;

        PORTB |= 0b00001110;
        _delay_us(20);

      for(i = 0; i < 2; i++){
        if(i == 0)temp01 = moduleNumber;
        if(i == 1)temp01 = moduleInstruction;
        if(temp01 & (1<<0)){
          PORTB &= 0b11111011;
        }
        else{
          PORTB &= 0b11111101;
          PORTB &= 0b11111001;
        }
        _delay_us(20);
        PORTB |= 0b00001110;
        _delay_us(20);

        if(temp01 & (1<<1)){
          PORTB &= 0b11111011;
        }
        else{
          PORTB &= 0b11111101;
          PORTB &= 0b11111001;
        }
        _delay_us(20);
        PORTB |= 0b00001110;
        _delay_us(20);

        if(temp01 & (1<<2)){
          PORTB &= 0b11111011;
        }
        else{
          PORTB &= 0b11111101;
          PORTB &= 0b11111001;
        }
        _delay_us(20);
        PORTB |= 0b00001110;
        _delay_us(20);

        if(temp01 & (1<<3)){
          PORTB &= 0b11111011;
        }
        else{
          PORTB &= 0b11111101;
          PORTB &= 0b11111001;
        }
        _delay_us(20);
        PORTB |= 0b00001110;
        _delay_us(20);

        if(temp01 & (1<<4)){
          PORTB &= 0b11111011;
        }
        else{
          PORTB &= 0b11111101;
          PORTB &= 0b11111001;
        }
        _delay_us(20);
        PORTB |= 0b00001110;
        _delay_us(20);

        if(temp01 & (1<<5)){
          PORTB &= 0b11111011;
        }
        else{
          PORTB &= 0b11111101;
          PORTB &= 0b11111001;
        }
        _delay_us(20);
        PORTB |= 0b00001110;
        _delay_us(20);

        if(temp01 & (1<<6)){
          PORTB &= 0b11111011;
        }
        else{
          PORTB &= 0b11111101;
          PORTB &= 0b11111001;
        }
        _delay_us(20);
        PORTB |= 0b00001110;
        _delay_us(20);

        if(temp01 & (1<<7)){
          PORTB &= 0b11111011;
        }
        else{
          PORTB &= 0b11111101;
          PORTB &= 0b11111001;
        }
        _delay_us(20);
        PORTB |= 0b00001110;
        _delay_us(20);
      }//for(i=0;i<3;i++){

      PORTB &= 0b11110001;
      sendBytes = 0;
      DDRB &= 0b11110001;
      PORTB &= 0b11110001;
    }//if(sendBytes == 1)

    if(PINB & (1 << 3)){//receive bytes from wired connection
      readInput();
      get01 = temp01;
      readInput();
      get02 = temp01;
      while(PINB & (1 << 3));
    }



    //Switch IR-LED to send module ID and to receive commands

    if(irLEDcounter==0){
      controlBits &= 0b11110001;
      moduleIRid = 0;
      PORTD &= 0b11101110;     //Turn off IR-LEDs and green signal LEDs
      if (irCommand == 15){
        headLight = headLight; //...or a more meaningful command...
      }
      if (irCommand == 20){
        headLight = headLight; //...or a more meaningful command...
      }
      irCommand = 0;
    }


    if(!(controlBits & (1 << 1))){
      controlBits &= 0b11110111; //Store current state of IR sensor
      if(!(PIND & (1 << 1))){    //Sensor is illuminated
        controlBits |= 0b00001000;
      }
      //Reset irLEDcounter to initial value if pulse is not long enough
      if(controlBits & (1 << 3)){
        if(!(PIND & (1 << 4))){
          PORTD |= 0b00010000;  //Turn on green signal LEDs
        }
        if(!(controlBits & (1 << 2))){
          controlBits |= 0b00000100;
          irLEDcounter = 70;
        }
      }

      if(!(controlBits & (1 << 3))){
        if(controlBits & (1 << 2)){
          controlBits &= 0b11111011;
          irLEDcounter = 70;
        }
      }

      if(controlBits & (1 << 3)){//Sensor is illuminated long enough to start communication
        if(irLEDcounter <= 7){
          controlBits |= 0b00000010;
          controlBits &= 0b11110011;
          irLEDcounter = 200;
        }
      }
    }//if(!(controlBits & (1 << 1)))

    if(controlBits & (1 << 1)){
      if(PIND & (1 << 1) && !(controlBits & (1 << 3))){ //Wait until sensor is not illuminated after long start pulse
        controlBits |= 0b00001000;  //Start sending IR-ID
        irLEDcounter = 20;
      }

      if(controlBits & (1 << 3)){
        if(moduleIRid < MODULE_ID_TO_SEND){//Send IR-ID
          if(irLEDcounter < 3){
            if(!(controlBits & (1 << 2))){
              controlBits |= 0b00000100;
              PORTD |= 0b00000001;  //Turn on IR LEDs
              irLEDcounter = 15;
            }
          }
          if(irLEDcounter < 9){
            if(controlBits & (1 << 2)){
              controlBits &= 0b11111011;
              PORTD &= 0b11111110;  //Turn off IR LEDs
              irLEDcounter = 30;
              moduleIRid++;
              if(moduleIRid == MODULE_ID_TO_SEND){
                irLEDcounter = 255;   //Wait some longer for commands after IR ID was sent
                controlBits |= 0b00000100;
              }
            }
          }
        }//if(moduleIRid < MODULE_ID_TO_SEND){//Send IR-ID
        if(moduleIRid == MODULE_ID_TO_SEND){//Get instructions
          if(!(PIND & (1 << 1))){
            if(!(controlBits & (1 << 2))){
              irCommand++;
              controlBits |= 0b00000100;
              irLEDcounter = 255;
            }
          }
          if(PIND & (1 << 1)){
            if(controlBits & (1 << 2)){
              controlBits &= 0b11111011;
              irLEDcounter = 255;
            }
          }
        }//if(moduleIRid == MODULE_ID_TO_SEND){//Get instructions
      }
    }

  }
  return 0;
}

